﻿using System.Text.Json;
using DAL;
using Microsoft.Extensions.Configuration;
using Shared.DTO.Integracion_DGA;
using Shared.DTO.VariablesEntorno;
using Shared.Helper;

namespace BLL.Integracion_DGA
{
    public class BusinessLogic
    {
        private readonly IConfiguration _configuration;
        private readonly ApiService _apiService;
        private readonly JobsDgaRepository _jobs;
        private readonly JobsDgaSupFlujRepository _jobsSupFluj;
        private readonly FileLoggerHelper _fileLoggerHelper;

        public BusinessLogic(IConfiguration configuration, ApiService apiService, JobsDgaRepository jobs, JobsDgaSupFlujRepository jobsSupFluj)
        {
            _configuration = configuration;
            _apiService = apiService;
            _jobs = jobs;
            _jobsSupFluj = jobsSupFluj;
            FileLoggerHelper.ConfigureLogger(_configuration);
        }

        public async Task Run()
        {

            DateTimeOffset dateEnd = DateTimeOffset.Now;
            DateTimeOffset dateStart = dateEnd.AddHours(-1);

            JsonSerializerOptions options = new JsonSerializerOptions() { PropertyNameCaseInsensitive = true };

            //FileLoggerHelper.ConfigureLogger(_configuration);
            WriteLineAndLog($"Inicia Proceso DGA");
            try
            {
                string apiUrlBase = NexusApiUrl.ApiUrl;
                Dictionary<string, string> headers = new Dictionary<string, string>
                  {
                      { "nexustoken", NexusApiUrl.ApiKey },
                      { "nexusapiversion", NexusApiUrl.Version },
                      { "accept", "application/json" }
                  };

                WriteLineAndLog($"Obteniendo Documentos");
                string apiUrlDocuments = apiUrlBase + "/api/Documents";

                // Utiliza el servicio para realizar la solicitud a la API con URL y encabezados personalizados
                string responseData = await _apiService.GetApiResponseAsync(apiUrlDocuments, headers);

                var documento = JsonSerializer.Deserialize<List<DocumentResponse>>(responseData, options);
                foreach (DocumentResponse item in documento)
                {
                    WriteLineAndLog($"Obteniendo Tagviews");
                    string apiUrlTagViews = $"{apiUrlBase}/api/Documents/tagviews/{item.uid}";
                    responseData = await _apiService.GetApiResponseAsync(apiUrlTagViews, headers);
                    TagviewsResponse tagviews = JsonSerializer.Deserialize<TagviewsResponse>(responseData, options);

                    List<string> listTagsID = tagviews.Columns.Select(x => x.Uid).ToList();

                    HistoricRequest historicRequest = new HistoricRequest
                    {
                        DataSource = NexusApiUrl.DataSource,
                        Resolution = NexusApiUrl.Resolution,
                        Uids = listTagsID,
                        StartTs = dateStart.ToUnixTimeSeconds(),
                        EndTs = dateEnd.ToUnixTimeSeconds()
                    };

                    WriteLineAndLog($"Obteniendo Tagviews Historic");
                    string apiUrlHistoric = $"{apiUrlBase}/api/Documents/tagviews/{item.uid}/realtime";
                    responseData = await _apiService.PostApiResponseAsync(apiUrlHistoric, headers, listTagsID);
                    List<HistoricResponse> historicResponse = JsonSerializer.Deserialize<List<HistoricResponse>>(responseData, options);

                    TimeZoneInfo zonaHorariaChile = TimeZoneInfo.FindSystemTimeZoneById("Pacific SA Standard Time");

                    if (item.name == "API - DGA - CAUDALES")
                    {
                        var caudalData = tagviews.Columns
                            .SelectMany(tag =>
                                historicResponse.Where(h => h.Uid == tag.Uid)
                                    .Select(h => new DGAInsert
                                    {
                                        TAG = tag.Name,
                                        VALOR = h.Value?.ToString(),
                                        FECHAMEDICION = TimeZoneInfo.ConvertTimeFromUtc(
                                            new DateTime(1970, 1, 1).AddSeconds(h.TimeStamp).ToUniversalTime(),
                                            zonaHorariaChile)
                                    })
                            ).ToList();

                        await _jobs.InsertarDgaCaudal(caudalData);
                    }
                    else if (item.name == "API - DGA - NIVELES")
                    {
                        var nivelData = tagviews.Columns
                            .SelectMany(tag =>
                                historicResponse.Where(h => h.Uid == tag.Uid)
                                    .Select(h => new DGAInsert
                                    {
                                        TAG = tag.Name,
                                        VALOR = h.Value?.ToString(),
                                        FECHAMEDICION = TimeZoneInfo.ConvertTimeFromUtc(
                                            new DateTime(1970, 1, 1).AddSeconds(h.TimeStamp).ToUniversalTime(),
                                            zonaHorariaChile)
                                    })
                            ).ToList();

                        await _jobs.InsertarDgaNivel(nivelData);
                    }
                    else if (item.name == "API - DGA")
                    {
                        var nivelData = tagviews.Columns
                            .SelectMany(tag =>
                                historicResponse.Where(h => h.Uid == tag.Uid)
                                    .Select(h => new
                                    {
                                        TAG = tag.Name,
                                        NIVEL_FREATICO = h.Value?.ToString(),
                                        FECHAMEDICION = TimeZoneInfo.ConvertTimeFromUtc(
                                            new DateTime(1970, 1, 1).AddSeconds(h.TimeStamp).ToUniversalTime(),
                                            zonaHorariaChile)
                                    })
                            ).ToList();
                    }
                }

                WriteLineAndLog($"INICIO REGISTRO DE MEDICIONES DGA");
                await _jobs.SpRegistrarMedicionesDga();
                WriteLineAndLog($"FIN REGISTRO DE MEDICIONES DGA");

                WriteLineAndLog($"FIN PROCESO DGA");
            }
            catch (Exception ex)
            {
                FileLoggerHelper.LogError($"{ex.Message}", ex);
                WriteLineAndLog($"{ex.Message}", ConsoleColor.Red);
            }

        }

        static void WriteLineAndLog(string msj, ConsoleColor? color = null)
        {
            if (color.HasValue && Enum.IsDefined(typeof(ConsoleColor), color.Value))
            {
                Console.ForegroundColor = (ConsoleColor)color;
                Console.WriteLine(msj);
                Console.ResetColor();
            }
            else
            {
                Console.WriteLine($"{msj}");
            }

            FileLoggerHelper.LogInformation($"{msj}");
        }
    }
}
